@@ -1,9 +1,10 @@
 /*
- * xmlversion.h : compile-time version informations for the XML parser.
+ * Summary: compile-time version informations
+ * Description: compile-time version informations for the XML library
  *
- * See Copyright for the status of this software.
+ * Copy: See Copyright for the status of this software.
  *
- * daniel@veillard.com
+ * Author: Daniel Veillard
  */
 
 #ifndef __XML_VERSION_H__
@@ -28,21 +29,21 @@ XMLPUBFUN void XMLCALL xmlCheckVersion(int version);
  *
  * the version string like "1.2.3"
  */
-#define LIBXML_DOTTED_VERSION "2.6.2"
+#define LIBXML_DOTTED_VERSION "2.6.3"
 
 /**
  * LIBXML_VERSION:
  *
  * the version number: 1.2.3 value is 1002003
  */
-#define LIBXML_VERSION 20602
+#define LIBXML_VERSION 20603
 
 /**
  * LIBXML_VERSION_STRING:
  *
  * the version number string, 1.2.3 value is "1002003"
  */
-#define LIBXML_VERSION_STRING "20602"
+#define LIBXML_VERSION_STRING "20603"
 
 /**
  * LIBXML_TEST_VERSION:
@@ -50,7 +51,7 @@ XMLPUBFUN void XMLCALL xmlCheckVersion(int version);
  * Macro to check that the libxml version in use is compatible with
  * the version the software has been compiled against
  */
-#define LIBXML_TEST_VERSION xmlCheckVersion(20602);
+#define LIBXML_TEST_VERSION xmlCheckVersion(20603);
 
 #ifndef VMS
 #if 0
@@ -69,6 +70,11 @@ XMLPUBFUN void XMLCALL xmlCheckVersion(int version);
 #define WITHOUT_TRIO
 #endif
 #else /* VMS */
+/**
+ * WITH_TRIO:
+ *
+ * defined if the trio support need to be configured in
+ */
 #define WITH_TRIO 1
 #endif /* VMS */
 
@@ -119,6 +125,15 @@ XMLPUBFUN void XMLCALL xmlCheckVersion(int version);
 #define LIBXML_READER_ENABLED
 #endif
 
+/**
+ * LIBXML_PATTERN_ENABLED:
+ *
+ * Whether the xmlPattern node selection interface is configured in
+ */
+#if 1
+#define LIBXML_PATTERN_ENABLED
+#endif
+
 /**
  * LIBXML_WRITER_ENABLED:
  *
@@ -318,7 +333,7 @@ XMLPUBFUN void XMLCALL xmlCheckVersion(int version);
 #include <ansidecl.h>
 #endif
 #ifndef ATTRIBUTE_UNUSED
-#define ATTRIBUTE_UNUSED
+#define ATTRIBUTE_UNUSED __attribute__((unused))
 #endif
 #else
 #define ATTRIBUTE_UNUSED
