@@ -1,4 +1,5 @@
 import libxml2mod
+import types
 
 #
 # Errors raised by the wrappers when some tree handling failed.
@@ -79,18 +80,37 @@ class ioReadWrapper(ioWrapper):
 
 class ioWriteWrapper(ioWrapper):
     def __init__(self, _obj, enc = ""):
-        ioWrapper.__init__(self, _obj)
-        self._o = libxml2mod.xmlCreateOutputBuffer(self, enc)
+#        print "ioWriteWrapper.__init__", _obj
+        if type(_obj) == type(''):
+	    print "write io from a string"
+	    self.o = None
+	elif type(_obj) == types.InstanceType:
+	    print "write io from instance of %s" % (_obj.__class__)
+	    ioWrapper.__init__(self, _obj)
+	    self._o = libxml2mod.xmlCreateOutputBuffer(self, enc)
+	else:
+	    file = libxml2mod.outputBufferGetPythonFile(_obj)
+	    if file != None:
+		ioWrapper.__init__(self, file)
+	    else:
+	        ioWrapper.__init__(self, _obj)
+	    self._o = _obj
 
     def __del__(self):
-        print "__del__"
+#        print "__del__"
         self.io_close()
         if self._o != None:
             libxml2mod.xmlOutputBufferClose(self._o)
         self._o = None
 
+    def flush(self):
+        self.io_flush()
+        if self._o != None:
+            libxml2mod.xmlOutputBufferClose(self._o)
+        self._o = None
+
     def close(self):
-        self.io_close()
+        self.io_flush()
         if self._o != None:
             libxml2mod.xmlOutputBufferClose(self._o)
         self._o = None
@@ -342,21 +362,25 @@ class xmlCore:
         ctxt.xpathFreeContext()
         return res
 
-    #
-    # Selecting nodes using XPath, faster because the context
-    # is allocated just once per xmlDoc.
-    #
+#    #
+#    # Selecting nodes using XPath, faster because the context
+#    # is allocated just once per xmlDoc.
+#    #
+#    # Removed: DV memleaks c.f. #126735
+#    #
+#    def xpathEval2(self, expr):
+#        doc = self.doc
+#        if doc == None:
+#            return None
+#        try:
+#            doc._ctxt.setContextNode(self)
+#        except:
+#            doc._ctxt = doc.xpathNewContext()
+#            doc._ctxt.setContextNode(self)
+#        res = doc._ctxt.xpathEval(expr)
+#        return res
     def xpathEval2(self, expr):
-        doc = self.doc
-        if doc == None:
-            return None
-        try:
-            doc._ctxt.setContextNode(self)
-        except:
-            doc._ctxt = doc.xpathNewContext()
-            doc._ctxt.setContextNode(self)
-        res = doc._ctxt.xpathEval(expr)
-        return res
+        return self.xpathEval(expr)
 
     # support for python2 iterators
     def walk_depth_first(self):
@@ -370,7 +394,7 @@ class xmlCore:
             self.doc._ctxt.xpathFreeContext()
         except:
             pass
-        libxml2mod.freeDoc(self._o)
+        libxml2mod.xmlFreeDoc(self._o)
 
 
 #
