@@ -29,19 +29,28 @@
 #include <ctype.h>
 #endif
 
+
 /**
  * MEM_LIST:
  *
  * keep track of all allocated blocks for error reporting 
  * Always build the memory list !
  */
+#ifdef DEBUG_MEMORY_LOCATION
 #ifndef MEM_LIST
 #define MEM_LIST /* keep a list of all the allocated memory blocks */
 #endif
+#endif
 
 #include <libxml/xmlmemory.h>
 #include <libxml/globals.h>
 #include <libxml/xmlerror.h>
+#include <libxml/threads.h>
+
+static int xmlMemInitialized = 0;
+static unsigned long  debugMemSize = 0;
+static unsigned long  debugMaxMemSize = 0;
+static xmlMutexPtr xmlMemMutex = NULL;
 
 void xmlMallocBreakpoint(void);
 
@@ -103,12 +112,9 @@ typedef struct memnod {
 #define HDR_2_CLIENT(a)    ((void *) (((char *) (a)) + RESERVE_SIZE))
 
 
-static unsigned long  debugMemSize = 0;
-static unsigned long  debugMaxMemSize = 0;
-static int block=0;
-static int xmlMemStopAtBlock = 0;
+static unsigned int block=0;
+static unsigned int xmlMemStopAtBlock = 0;
 static void *xmlMemTraceBlockAt = NULL;
-static int xmlMemInitialized = 0;
 #ifdef MEM_LIST
 static MEMHDR *memlist = NULL;
 #endif
@@ -172,23 +178,25 @@ xmlMallocLoc(size_t size, const char * file, int line)
 	return(NULL);
     }   
     p->mh_tag = MEMTAG;
-    p->mh_number = ++block;
     p->mh_size = size;
     p->mh_type = MALLOC_TYPE;
     p->mh_file = file;
     p->mh_line = line;
+    xmlMutexLock(xmlMemMutex);
+    p->mh_number = ++block;
     debugMemSize += size;
     if (debugMemSize > debugMaxMemSize) debugMaxMemSize = debugMemSize;
 #ifdef MEM_LIST
     debugmem_list_add(p);
 #endif
-
+    xmlMutexUnlock(xmlMemMutex);
+    
 #ifdef DEBUG_MEMORY
     xmlGenericError(xmlGenericErrorContext,
 	    "Malloc(%d) Ok\n",size);
 #endif
     
-    if (xmlMemStopAtBlock == block) xmlMallocBreakpoint();
+    if (xmlMemStopAtBlock == p->mh_number) xmlMallocBreakpoint();
 
     ret = HDR_2_CLIENT(p);
 
@@ -237,23 +245,25 @@ xmlMallocAtomicLoc(size_t size, const char * file, int line)
 	return(NULL);
     }   
     p->mh_tag = MEMTAG;
-    p->mh_number = ++block;
     p->mh_size = size;
     p->mh_type = MALLOC_ATOMIC_TYPE;
     p->mh_file = file;
     p->mh_line = line;
+    xmlMutexLock(xmlMemMutex);
+    p->mh_number = ++block;
     debugMemSize += size;
     if (debugMemSize > debugMaxMemSize) debugMaxMemSize = debugMemSize;
 #ifdef MEM_LIST
     debugmem_list_add(p);
 #endif
+    xmlMutexUnlock(xmlMemMutex);
 
 #ifdef DEBUG_MEMORY
     xmlGenericError(xmlGenericErrorContext,
 	    "Malloc(%d) Ok\n",size);
 #endif
     
-    if (xmlMemStopAtBlock == block) xmlMallocBreakpoint();
+    if (xmlMemStopAtBlock == p->mh_number) xmlMallocBreakpoint();
 
     ret = HDR_2_CLIENT(p);
 
@@ -312,11 +322,13 @@ xmlReallocLoc(void *ptr,size_t size, const char * file, int line)
 	 goto error;
     }
     p->mh_tag = ~MEMTAG;
+    xmlMutexLock(xmlMemMutex);
     debugMemSize -= p->mh_size;
 #ifdef MEM_LIST
     debugmem_list_delete(p);
 #endif
-
+    xmlMutexUnlock(xmlMemMutex);
+    
     p = (MEMHDR *) realloc(p,RESERVE_SIZE+size);
     if (!p) {
 	 goto error;
@@ -333,11 +345,13 @@ xmlReallocLoc(void *ptr,size_t size, const char * file, int line)
     p->mh_size = size;
     p->mh_file = file;
     p->mh_line = line;
+    xmlMutexLock(xmlMemMutex);
     debugMemSize += size;
     if (debugMemSize > debugMaxMemSize) debugMaxMemSize = debugMemSize;
 #ifdef MEM_LIST
     debugmem_list_add(p);
 #endif
+    xmlMutexUnlock(xmlMemMutex);
 
     TEST_POINT
 
@@ -396,12 +410,14 @@ xmlMemFree(void *ptr)
         goto error;
     }
     p->mh_tag = ~MEMTAG;
-    debugMemSize -= p->mh_size;
     memset(target, -1, p->mh_size);
-
+    xmlMutexLock(xmlMemMutex);
+    debugMemSize -= p->mh_size;
 #ifdef MEM_LIST
     debugmem_list_delete(p);
 #endif
+    xmlMutexUnlock(xmlMemMutex);
+
     free(p);
 
     TEST_POINT
@@ -441,19 +457,22 @@ xmlMemStrdupLoc(const char *str, const char *file, int line)
       goto error;
     }
     p->mh_tag = MEMTAG;
-    p->mh_number = ++block;
     p->mh_size = size;
     p->mh_type = STRDUP_TYPE;
     p->mh_file = file;
     p->mh_line = line;
+    xmlMutexLock(xmlMemMutex);
+    p->mh_number = ++block;
     debugMemSize += size;
     if (debugMemSize > debugMaxMemSize) debugMaxMemSize = debugMemSize;
 #ifdef MEM_LIST
     debugmem_list_add(p);
 #endif
+    xmlMutexUnlock(xmlMemMutex);
+    
     s = (char *) HDR_2_CLIENT(p);
     
-    if (xmlMemStopAtBlock == block) xmlMallocBreakpoint();
+    if (xmlMemStopAtBlock == p->mh_number) xmlMallocBreakpoint();
 
     if (s != NULL)
       strcpy(s,str);
@@ -559,52 +578,6 @@ xmlMemContentShow(FILE *fp, MEMHDR *p)
 }
 #endif
 
-/**
- * xmlMemShow:
- * @fp:  a FILE descriptor used as the output file
- * @nr:  number of entries to dump
- *
- * show a show display of the memory allocated, and dump
- * the @nr last allocated areas which were not freed
- */
-
-void
-xmlMemShow(FILE *fp, int nr)
-{
-#ifdef MEM_LIST
-    MEMHDR *p;
-#endif
-
-    if (fp != NULL)
-	fprintf(fp,"      MEMORY ALLOCATED : %lu, MAX was %lu\n",
-		debugMemSize, debugMaxMemSize);
-#ifdef MEM_LIST
-    if (nr > 0) {
-	fprintf(fp,"NUMBER   SIZE  TYPE   WHERE\n");
-	p = memlist;
-	while ((p) && nr > 0) {
-	      fprintf(fp,"%6lu %6lu ",p->mh_number,(unsigned long)p->mh_size);
-	    switch (p->mh_type) {
-	       case STRDUP_TYPE:fprintf(fp,"strdup()  in ");break;
-	       case MALLOC_TYPE:fprintf(fp,"malloc()  in ");break;
-	       case MALLOC_ATOMIC_TYPE:fprintf(fp,"atomicmalloc()  in ");break;
-	      case REALLOC_TYPE:fprintf(fp,"realloc() in ");break;
-	      case REALLOC_ATOMIC_TYPE:fprintf(fp,"atomicrealloc() in ");break;
-		default:fprintf(fp,"   ???    in ");break;
-	    }
-	    if (p->mh_file != NULL)
-	        fprintf(fp,"%s(%d)", p->mh_file, p->mh_line);
-	    if (p->mh_tag != MEMTAG)
-		fprintf(fp,"  INVALID");
-	    xmlMemContentShow(fp, p);
-	    fprintf(fp,"\n");
-	    nr--;
-	    p = p->mh_next;
-	}
-    }
-#endif /* MEM_LIST */    
-}
-
 /**
  * xmlMemDisplay:
  * @fp:  a FILE descriptor used as the output file, if NULL, the result is
@@ -636,6 +609,7 @@ xmlMemDisplay(FILE *fp)
             debugMemSize, debugMaxMemSize);
     fprintf(fp,"BLOCK  NUMBER   SIZE  TYPE\n");
     idx = 0;
+    xmlMutexLock(xmlMemMutex);
     p = memlist;
     while (p) {
 	  fprintf(fp,"%-5u  %6lu %6lu ",idx++,p->mh_number,
@@ -660,6 +634,7 @@ xmlMemDisplay(FILE *fp)
         fprintf(fp,"\n");
         p = p->mh_next;
     }
+    xmlMutexUnlock(xmlMemMutex);
 #else
     fprintf(fp,"Memory list not compiled (MEM_LIST not defined !)\n");
 #endif
@@ -710,8 +685,57 @@ static void debugmem_tag_error(void *p)
 #endif
 }
 
+#ifdef MEM_LIST
 static FILE *xmlMemoryDumpFile = NULL;
+#endif
 
+/**
+ * xmlMemShow:
+ * @fp:  a FILE descriptor used as the output file
+ * @nr:  number of entries to dump
+ *
+ * show a show display of the memory allocated, and dump
+ * the @nr last allocated areas which were not freed
+ */
+
+void
+xmlMemShow(FILE *fp, int nr ATTRIBUTE_UNUSED)
+{
+#ifdef MEM_LIST
+    MEMHDR *p;
+#endif
+
+    if (fp != NULL)
+	fprintf(fp,"      MEMORY ALLOCATED : %lu, MAX was %lu\n",
+		debugMemSize, debugMaxMemSize);
+#ifdef MEM_LIST
+    xmlMutexLock(xmlMemMutex);
+    if (nr > 0) {
+	fprintf(fp,"NUMBER   SIZE  TYPE   WHERE\n");
+	p = memlist;
+	while ((p) && nr > 0) {
+	      fprintf(fp,"%6lu %6lu ",p->mh_number,(unsigned long)p->mh_size);
+	    switch (p->mh_type) {
+	       case STRDUP_TYPE:fprintf(fp,"strdup()  in ");break;
+	       case MALLOC_TYPE:fprintf(fp,"malloc()  in ");break;
+	       case MALLOC_ATOMIC_TYPE:fprintf(fp,"atomicmalloc()  in ");break;
+	      case REALLOC_TYPE:fprintf(fp,"realloc() in ");break;
+	      case REALLOC_ATOMIC_TYPE:fprintf(fp,"atomicrealloc() in ");break;
+		default:fprintf(fp,"   ???    in ");break;
+	    }
+	    if (p->mh_file != NULL)
+	        fprintf(fp,"%s(%d)", p->mh_file, p->mh_line);
+	    if (p->mh_tag != MEMTAG)
+		fprintf(fp,"  INVALID");
+	    xmlMemContentShow(fp, p);
+	    fprintf(fp,"\n");
+	    nr--;
+	    p = p->mh_next;
+	}
+    }
+    xmlMutexUnlock(xmlMemMutex);
+#endif /* MEM_LIST */    
+}
 
 /**
  * xmlMemoryDump:
@@ -722,6 +746,7 @@ static FILE *xmlMemoryDumpFile = NULL;
 void
 xmlMemoryDump(void)
 {
+#ifdef MEM_LIST
     FILE *dump;
 
     if (debugMaxMemSize == 0)
@@ -734,6 +759,7 @@ xmlMemoryDump(void)
     xmlMemDisplay(xmlMemoryDumpFile);
 
     if (dump != NULL) fclose(dump);
+#endif /* MEM_LIST */
 }
 
 
@@ -755,18 +781,18 @@ static int xmlInitMemoryDone = 0;
 int
 xmlInitMemory(void)
 {
-     int ret;
-     
 #ifdef HAVE_STDLIB_H
      char *breakpoint;
 #endif     
 
+     xmlMemInitialized = 1;
      if (xmlInitMemoryDone) return(-1);
+     xmlMemMutex = xmlNewMutex();
 
 #ifdef HAVE_STDLIB_H
      breakpoint = getenv("XML_MEM_BREAKPOINT");
      if (breakpoint != NULL) {
-         sscanf(breakpoint, "%d", &xmlMemStopAtBlock);
+         sscanf(breakpoint, "%ud", &xmlMemStopAtBlock);
      }
 #endif     
 #ifdef HAVE_STDLIB_H
@@ -780,11 +806,9 @@ xmlInitMemory(void)
      xmlGenericError(xmlGenericErrorContext,
 	     "xmlInitMemory() Ok\n");
 #endif     
-     xmlMemInitialized = 1;
      xmlInitMemoryDone = 1;
 
-     ret = 0;
-     return(ret);
+     return(0);
 }
 
 /**
